#import "BrokenBox.h"

#import "Physics.h"
#import "BreakableBox.h"

@implementation BrokenBox

static float frand_unit(){ return (rand()/(float)RAND_MAX)*2.0f - 1.0f; }

struct Chunk {
	ChipmunkCocosBody *body;
	ChipmunkShape *shape;
	CCSprite *sprite;
};

static struct Chunk MakeChunk(BreakableBox *box, cpVect direction){
		ChipmunkBody *boxBody = box.body;
		CGRect boxRect = box.sprite.textureRect;
		CGFloat hw = boxRect.size.width/2.0f;
		CGFloat hh = boxRect.size.height/2.0f;
		
		cpFloat mass = boxBody.mass/4.0f;
    ChipmunkCocosBody *body = [ChipmunkCocosBody bodyWithMass:mass andMoment:cpMomentForBox(mass, hw, hh)];
    body.pos = [boxBody local2world:cpv(direction.x*hw/2.0f, direction.y*hh/2.0f)];
		body.angle = boxBody.angle;
		body.vel = cpvadd(boxBody.vel, cpvrotate(cpvmult(direction, 50), boxBody.rot));
		body.angVel = boxBody.angVel + frand_unit()*2.0f;
    
    ChipmunkShape * shape = [ChipmunkPolyShape boxWithBody:body width:hw height:hh];
		shape.elasticity = 0.3f;
		shape.friction = 0.8f;
		shape.layers = PhysicsLayerDebris;
		shape.group = [BrokenBox class]; // debris should not collide with debris
    
		// Note that Cocos2D texture y-coords are flipped
		CGRect rect = CGRectMake(boxRect.origin.x + (direction.x*0.5f + 0.5f)*hw, boxRect.origin.y + (0.5f - direction.y*0.5f)*hh, hw, hh);
		CCSprite *sprite = [CCSprite spriteWithBatchNode:box.sprite.batchNode rect:rect];
		
		// Connect the sprite to the body.
		body.syncedNodes = [NSArray arrayWithObject:sprite];
		
		return (struct Chunk){body, shape, sprite};
}

-(id)initFromBox:(BreakableBox *)box;
{
	if((self = [super init])){
		struct Chunk a = MakeChunk(box, cpv(-1.0f, -1.0f));
		struct Chunk b = MakeChunk(box, cpv( 1.0f, -1.0f));
		struct Chunk c = MakeChunk(box, cpv(-1.0f,  1.0f));
		struct Chunk d = MakeChunk(box, cpv( 1.0f,  1.0f));
		
		// Same as before, build out the three arrays and all the adding/removing will be done automagically!
    chipmunkObjects = [[NSArray alloc] initWithObjects:a.body, b.body, c.body, d.body, a.shape, b.shape, c.shape, d.shape, nil];
    sprites = [[NSArray alloc] initWithObjects:a.sprite, b.sprite, c.sprite, d.sprite, nil];
		
		[self fadeOut:1.0f];
	}
	
	return self;
}

@end
